'use strict';
import { atajoXPayload } from "../payload/payload";
import { atajoXConfig } from "../config/config";
import { Observable } from "rxjs/Observable";
import { atajoXEvents } from "../events/events";
import { atajoXSocket } from "../socket/socket";
import { atajoXQueue } from "../queue/queue";
var AtajoXApi = /** @class */ (function () {
    function AtajoXApi() {
        atajoXEvents.subscribe('socket:client:rx', AtajoXApi.transactionResponse);
    }
    /**
     * Add transaction to be processed
     *
     * @param {string} lambda
     * @param {object} request
     * @param opts
     * @returns {Observable<any>}
     */
    AtajoXApi.prototype.addTransaction = function (lambda, request, opts) {
        if (opts === void 0) { opts = {}; }
        return Observable.create(function (observer) {
            atajoXPayload.getPayload(request).then(function (payload) {
                var timeout = opts['timeout'] || atajoXConfig.STATE.api.timeout;
                var queueOpt = opts['queue'] || false;
                var pid = payload['pid'];
                var lambdaGroup = (lambda[0] === '@') ? '' : atajoXConfig.STATE.lambdaGroup;
                var lambdaPath = lambdaGroup + lambda;
                var transaction = {
                    timeout: timeout,
                    latency: {
                        deviceSubmitAt: new Date().getTime(),
                        deviceRequestAt: 0,
                        deviceQueuedAt: 0
                    },
                    payload: payload,
                    at: new Date().getTime(),
                    status: 'SUBMIT',
                    observer: observer,
                    lambda: lambdaPath,
                    queue: queueOpt,
                    version: atajoXConfig.STATE.consulConfig.core[atajoXConfig.STATE.environment].version,
                    environment: atajoXConfig.STATE.environment
                };
                if (!queueOpt && !atajoXSocket.isConnected()) {
                    observer.error('Device Not Connected');
                }
                else if (queueOpt) {
                    atajoXQueue.add(pid, transaction);
                    observer.next({ pid: pid });
                    observer.complete();
                }
                else {
                    AtajoXApi.transactions[pid] = transaction;
                    AtajoXApi.startTimeoutCheck();
                    AtajoXApi.processTransaction(pid);
                }
            }, function () {
                observer.error('Could not get payload');
            });
        });
    };
    /**
     * Get all transactions
     *
     * @returns {ApiTransactions}
     */
    AtajoXApi.getTransactions = function () {
        return AtajoXApi.transactions;
    };
    /**
     * Process transaction response
     *
     * When the transaction comes back from the lambdas, process it.
     *
     * @param rx
     */
    AtajoXApi.transactionResponse = function (rx) {
        if (rx.pid) {
            var pid = rx.pid;
            var transaction = AtajoXApi.transactions[pid];
            if (transaction) {
                var observer = transaction.observer;
                if (rx.hasOwnProperty('response') === false) {
                    observer.error({ progress: 0, data: rx });
                }
                //CALC PERCENTAGES FROM CHUNKS
                var chunkId = rx.response.chunkId + 1;
                var chunkTotal = rx.response.chunkTotal;
                var progress = (100 / chunkTotal) * chunkId;
                progress = isNaN(progress) ? 0 : progress;
                if (!rx.response.data)
                    rx.response.data = [];
                if (Array.isArray(rx.response.data)) {
                    if (rx.response.data.length == 1) {
                        rx.response.data = rx.response.data[0];
                    }
                }
                if (rx.error) {
                    observer.error({ progress: progress, data: rx.response.data });
                }
                else {
                    observer.next({ progress: progress, data: rx.response.data });
                }
                delete AtajoXApi.transactions[pid];
                observer.complete();
            }
        }
        else {
            console.error("SOCKET:TX_RESPONSE INVALID : ", rx);
        }
    };
    /**
     * Process transaction that is not queued
     *
     * @param {string} pid
     */
    AtajoXApi.processTransaction = function (pid) {
        var transaction = AtajoXApi.transactions[pid];
        //MAKE THE CALL;
        transaction.latency = transaction.latency || { deviceSubmitAt: 0, deviceRequestAt: 0, deviceQueuedAt: 0 };
        transaction.latency.deviceRequestAt = new Date().getTime();
        atajoXSocket.transaction(transaction);
    };
    /**
     * Checks timeouts on transactions
     */
    AtajoXApi.checkTimeouts = function () {
        if (Object.keys(AtajoXApi.transactions).length == 0) {
            AtajoXApi.stopTimeoutCheck();
            return;
        }
        for (var pid in AtajoXApi.transactions) {
            var tx = AtajoXApi.transactions[pid];
            var now = new Date().getTime();
            if ((now - tx.at) > tx.timeout) {
                console.debug("TX FOR PID " + pid + " TIMED OUT");
                tx.observer.error({ progress: 0, data: "Transaction Timed Out" });
                delete AtajoXApi.transactions[pid];
            }
        }
    };
    /**
     *  Starts interval check for timeouts
     */
    AtajoXApi.startTimeoutCheck = function () {
        if (!AtajoXApi.timeoutCheckInterval) {
            AtajoXApi.timeoutCheckInterval = setInterval(AtajoXApi.checkTimeouts, 1000);
        }
    };
    /**
     * Stops interval for timeout check
     */
    AtajoXApi.stopTimeoutCheck = function () {
        if (AtajoXApi.timeoutCheckInterval) {
            clearInterval(AtajoXApi.timeoutCheckInterval);
        }
        AtajoXApi.timeoutCheckInterval = false;
    };
    AtajoXApi.transactions = {};
    return AtajoXApi;
}());
export { AtajoXApi };
export var atajoXApi = new AtajoXApi();
//# sourceMappingURL=api.js.map