'use strict';
var AtajoXSQLite = /** @class */ (function () {
    function AtajoXSQLite() {
    }
    /**
     * Returns database instance
     *
     * @returns {any}
     */
    AtajoXSQLite.prototype.getDb = function () {
        return this.db;
    };
    /**
     * Initiates database connection
     *
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.init = function () {
        var that = this;
        return new Promise(function (resolve, reject) {
            // document.addEventListener('deviceready', function () {
            try {
                that.db = window.sqlitePlugin.openDatabase({ name: 'atajo.client.db', location: 'default' });
                resolve();
            }
            catch (e) {
                reject('Can not create SQLite database on web: {e}');
            }
            // });
        });
    };
    /**
     * Get data from table
     *
     * @param {string} table
     * @param {string} where
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.get = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
            _this.db.transaction(function (tr) {
                tr.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, {}, function (tx, resultSet) {
                    var rows = [];
                    var len = resultSet.rows.length;
                    for (var i = 0; i < len; ++i) {
                        var obj = resultSet.rows.item(i);
                        var processedRow = {};
                        for (var r in obj) {
                            if (obj.hasOwnProperty(r)) {
                                processedRow[r] = obj[r];
                            }
                        }
                        rows.push(processedRow);
                    }
                    resolve(rows);
                }, function (tx, error) {
                    console.error(error);
                    reject(error);
                });
            });
        });
    };
    /**
     * Create table in database
     *
     * @param {string} table
     * @param {Array<any>} columns
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.create = function (table, columns) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var columnsWithTypes = [];
                columns.forEach(function (column) {
                    if (column.type.toUpperCase() == "BOOLEAN")
                        column.type = "INTEGER";
                    columnsWithTypes.push(column.name + ' ' + column.type);
                });
                var query = 'CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')';
                tx.executeSql(query, {}, function (tx, resultSet) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Save data to table
     *
     * @param {string} table
     * @param values
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.set = function (table, values) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (values.length == 0) {
                resolve();
            }
            else {
                var columns_1 = {};
                var valuePlaceholders_1 = [];
                var rowColumns = [];
                values.forEach(function (value) {
                    for (var v in value) {
                        if (value.hasOwnProperty(v)) {
                            columns_1[v] = true;
                        }
                    }
                });
                for (var i_1 in columns_1) {
                    valuePlaceholders_1.push('?');
                    rowColumns.push(i_1);
                }
                var columnCollection_1 = '(' + rowColumns.join(', ') + ')';
                var chunks = [];
                var i, j, chunk = 50;
                for (i = 0, j = values.length; i < j; i += chunk) {
                    chunks.push(values.slice(i, i + chunk));
                }
                var queryCollection_1 = [];
                chunks.forEach(function (chunk) {
                    var placeHolderCollection = [];
                    var rowValueCollection = [];
                    chunk.forEach(function (value) {
                        var rowValues = [];
                        for (var v in columns_1) {
                            if (value.hasOwnProperty(v)) {
                                if (value[v] && (value[v].constructor === Array || value[v].constructor === Object)) {
                                    rowValues.push(JSON.stringify(value[v]));
                                }
                                else if (typeof value[v] === "boolean") {
                                    rowValues.push(value[v] ? 1 : 0);
                                }
                                else {
                                    rowValues.push(value[v]);
                                }
                            }
                            else {
                                rowValues.push(null);
                            }
                        }
                        placeHolderCollection.push('(' + valuePlaceholders_1.join(', ') + ')');
                        rowValueCollection = rowValueCollection.concat(rowValues);
                    });
                    var query = 'INSERT INTO ' + table + ' ' + columnCollection_1 + ' VALUES ' + placeHolderCollection.join(', ');
                    queryCollection_1.push({
                        query: query,
                        rowValueCollection: rowValueCollection,
                    });
                });
                var that_1 = _this;
                (function insertRows(i) {
                    if (i < queryCollection_1.length) {
                        that_1.db.transaction(function (tx) {
                            tx.executeSql(queryCollection_1[i].query, queryCollection_1[i].rowValueCollection, function (tx, resultSet) {
                                insertRows(++i);
                            }, function (tx, error) {
                                console.error(error);
                                reject();
                            });
                        });
                    }
                    else {
                        resolve();
                    }
                })(0);
            }
        });
    };
    /**
     * Update row(s) in table
     *
     * @param {string} table
     * @param values
     * @param where
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.update = function (table, values, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var setColumns = [];
            var whereColumns = [];
            var rowValues = [];
            values.forEach(function (value) {
                setColumns.push(value.column + '=?');
                if (value.value && (value.value.constructor === Array || value.value.constructor === Object)) {
                    rowValues.push(JSON.stringify(value.value));
                }
                else {
                    rowValues.push(value.value);
                }
            });
            if (where) {
                where.forEach(function (value) {
                    whereColumns.push(value.column + (value.op || '=') + '?');
                    if (value.value && (value.value.constructor === Array || value.value.constructor === Object)) {
                        rowValues.push(JSON.stringify(value.value));
                    }
                    else {
                        rowValues.push(value.value);
                    }
                });
            }
            var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + whereColumns.join(', ') : '';
            var query = 'UPDATE ' + table + ' SET ' + setColumns.join(', ') + ' ' + whereStatement + '';
            _this.db.transaction(function (tx) {
                tx.executeSql(query, rowValues, function (tx, resultSet) {
                    resolve(resultSet);
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Returns the row count
     *
     * @param {string} table
     * @param {string} where
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.count = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
            _this.db.transaction(function (tx) {
                tx.executeSql('SELECT COUNT(*) FROM ' + table + ' ' + whereStatement, {}, function (tx, resultSet) {
                    if (resultSet.rows.length == 1) {
                        resolve(resultSet.rows.item(0)['COUNT(*)']);
                    }
                    else {
                        reject('no rows returned');
                    }
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Remove row(s) from table
     *
     * @param {string} table
     * @param {string} where
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.remove = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
            _this.db.transaction(function (tx) {
                tx.executeSql('DELETE FROM ' + table + ' ' + whereStatement, {}, function (tx, resultSet) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Deletes table from database
     *
     * @param {string} table
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.nuke = function (table) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                tx.executeSql('DROP TABLE IF EXISTS ' + table, {}, function (tx, resultSet) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Adds a column to database
     *
     * @param {string} table
     * @param {any} column
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.addColumn = function (table, column) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                tx.executeSql('ALTER TABLE ' + table + ' ADD COLUMN ' + column.name + ' ' + column.type, {}, function (tx, resultSet) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Adds a index to database
     *
     * @param {string} table
     * @param {any} column
     * @returns {Promise<any>}
     */
    AtajoXSQLite.prototype.addIndex = function (table, column) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var query = 'CREATE INDEX ' + table + '_' + column + '_index ON ' + table + ' (' + column + ')';
                tx.executeSql(query);
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    return AtajoXSQLite;
}());
export { AtajoXSQLite };
export var atajoXSQLite = new AtajoXSQLite();
//# sourceMappingURL=sqlite.js.map