'use strict';
var AtajoXWebSql = /** @class */ (function () {
    function AtajoXWebSql() {
        this.db = null;
    }
    /**
     * Initiates instance of database
     *
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.init = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db = window.openDatabase("atajo.client", "1.0", "Atajo Client Database", 1000000);
            resolve();
        });
    };
    /**
     * Returns the row count
     *
     * @param {string} table
     * @param {string} where
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.count = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                tx.executeSql('SELECT COUNT(*) FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
                    if (rs.rows.length == 1) {
                        resolve(rs.rows[0]['COUNT(*)']);
                    }
                    else {
                        reject('no rows returned');
                    }
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Creates new table in database
     *
     * @param {string} table
     * @param {Array<any>} columns
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.create = function (table, columns) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var columnsWithTypes = [];
                columns.forEach(function (column) {
                    columnsWithTypes.push(column.name + ' ' + column.type);
                });
                var query = 'CREATE TABLE IF NOT EXISTS ' + table + ' (' + columnsWithTypes.join(', ') + ')';
                tx.executeSql(query);
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    /**
     * Get data from table
     *
     * @param {string} table
     * @param {string} where
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.get = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                tx.executeSql('SELECT * FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
                    var rows = [];
                    var len = rs.rows.length;
                    for (var i = 0; i < len; ++i) {
                        var obj = rs.rows.item(i);
                        var processedRow = {};
                        for (var r in obj) {
                            if (obj.hasOwnProperty(r)) {
                                processedRow[r] = obj[r];
                            }
                        }
                        rows.push(processedRow);
                    }
                    resolve(rows);
                }, function (tx, error) {
                    console.error(error);
                    reject(error);
                });
            });
        });
    };
    /**
     * Save data to table
     *
     * @param table
     * @param {Array<any>} values
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.set = function (table, values) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                values.forEach(function (value) {
                    var rowColumns = [];
                    var rowValues = [];
                    var valuePlaceholders = [];
                    for (var v in value) {
                        if (value.hasOwnProperty(v)) {
                            rowColumns.push(v);
                            if (value[v] && (value[v].constructor === Array || value[v].constructor === Object)) {
                                rowValues.push(JSON.stringify(value[v]));
                            }
                            else {
                                rowValues.push(value[v]);
                            }
                        }
                    }
                    rowColumns.forEach(function () {
                        valuePlaceholders.push('?');
                    });
                    var query = 'INSERT INTO ' + table + ' (' + rowColumns.join(', ') + ') VALUES (' + valuePlaceholders.join(', ') + ')';
                    tx.executeSql(query, rowValues);
                });
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    /**
     * Remove row(s) from table
     *
     * @param {string} table
     * @param {string} where
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.remove = function (table, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + where : '';
                tx.executeSql('DELETE FROM ' + table + ' ' + whereStatement, [], function (tx, rs) {
                    resolve();
                }, function (tx, error) {
                    console.error(error);
                    reject();
                });
            });
        });
    };
    /**
     * Delete table from database
     *
     * @param {string} table
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.nuke = function (table) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            try {
                _this.db.transaction(function (tx) {
                    tx.executeSql('DROP TABLE IF EXISTS ' + table, [], function (tx, rs) {
                        resolve();
                    }, function (tx, error) {
                        console.error(error);
                        reject();
                    });
                });
            }
            catch (err) {
                console.error(err);
                reject();
            }
        });
    };
    /**
     * Update row(s) in table
     *
     * @param {string} table
     * @param {Array<any>} values
     * @param {Array<any>} where
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.update = function (table, values, where) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var setColumns = [];
                var whereColumns = [];
                var rowValues = [];
                values.forEach(function (value) {
                    setColumns.push(value.column + '=?');
                    if (value.value && (value.value.constructor === Array || value.value.constructor === Object)) {
                        rowValues.push(JSON.stringify(value.value));
                    }
                    else {
                        rowValues.push(value.value);
                    }
                });
                if (where) {
                    where.forEach(function (value) {
                        whereColumns.push(value.column + (value.op || '=') + '?');
                        if (value.value && (value.value.constructor === Array || value.value.constructor === Object)) {
                            rowValues.push(JSON.stringify(value.value));
                        }
                        else {
                            rowValues.push(value.value);
                        }
                    });
                }
                var whereStatement = (typeof where !== "undefined") ? 'WHERE ' + whereColumns.join(', ') : '';
                var query = 'UPDATE ' + table + ' SET ' + setColumns.join(', ') + ' ' + whereStatement + '';
                tx.executeSql(query, rowValues);
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    /**
     * Adds a column to database
     *
     * @param {string} table
     * @param {any} column
     * @returns {Promise<any>}
     */
    AtajoXWebSql.prototype.addColumn = function (table, column) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.db.transaction(function (tx) {
                var query = 'ALTER TABLE ' + table + ' ADD COLUMN ' + column.name + ' ' + column.type;
                tx.executeSql(query);
            }, function (error) {
                console.error(error);
                reject();
            }, function () {
                resolve();
            });
        });
    };
    return AtajoXWebSql;
}());
export { AtajoXWebSql };
export var atajoXWebSql = new AtajoXWebSql();
//# sourceMappingURL=websql.js.map