'use strict';
var AtajoXGraph = /** @class */ (function () {
    function AtajoXGraph(options) {
        this.libName = 'ReportingGraph';
        this.availableChartTypes = ['line', 'ring', 'bar', 'bar_horizontal'];
        this.ringTruePercentage = null;
        this.steps = 0;
        this._setDefaultOptions();
        this._setOptions(options);
        this._setup();
    }
    /**
     * Get graph options
     *
     * @returns {iOptions}
     */
    AtajoXGraph.prototype.getOptions = function () {
        return this._options;
    };
    /**
     * Show error in console
     *
     * @param {string} msg
     */
    AtajoXGraph.prototype.error = function (msg) {
        console.error('[' + this.libName + '] ' + msg);
    };
    /**
     * Set graph options
     *
     * @param options
     * @private
     */
    AtajoXGraph.prototype._setOptions = function (options) {
        options = (typeof options === 'undefined') ? {} : options;
        this._checkOptionErrors(options);
        for (var key in options) {
            if (options.hasOwnProperty(key)) {
                if (this.availableChartTypes.indexOf(key) === -1) {
                    this._options[key] = options[key];
                }
                else {
                    for (var graphKey in options[key]) {
                        if (options[key].hasOwnProperty(graphKey)) {
                            this._options[key][graphKey] = options[key][graphKey];
                        }
                    }
                }
            }
        }
    };
    /**
     * Run setup to initiate graph
     *
     * @private
     */
    AtajoXGraph.prototype._setup = function () {
        this._c = document.getElementById(this._options.id);
        if (this._c.height === 150 && this._c.width === 300 && this._c.parentElement) {
            var offsetWidth = this._c.parentElement.offsetWidth;
            this._c.height = offsetWidth / 2;
            this._c.width = offsetWidth;
        }
        if (this._options.type === 'bar_horizontal') {
            this._c.height = this._options.bar_horizontal.height;
        }
        this._ctx = this._c.getContext('2d');
        this._ratio = this._c.width / this._c.height;
        this._gutter = 10 * this._ratio;
        this._nudge = 20 * this._ratio;
        this._c.style.backgroundColor = this._options.background_color;
        switch (this._options.type) {
            case 'line':
                this._axisSetup();
                this._drawLineChart();
                this._animate();
                break;
            case 'ring':
                this._drawRingChart();
                this._animate();
                break;
            case 'bar':
                this._axisSetup();
                this._drawBarChart();
                this._animate();
                break;
            case 'bar_horizontal':
                this._drawBarHorizontalChart();
                this._animate();
                break;
        }
    };
    /**
     * Set default graph options
     *
     * @private
     */
    AtajoXGraph.prototype._setDefaultOptions = function () {
        this._options = {
            'id': 'graph',
            'type': 'line',
            'title': '',
            'series': [],
            'data': [],
            'percentage': 0,
            'colors': ['#055d8d', '#1279be'],
            'background_color': '#ffffff',
            'animate': true,
            'line': {
                'title_size': 18,
                'grid': true,
                'grid_series': true,
                'grid_fill': true,
                'grid_line_width': 1,
                'grid_color': '#8f8f8f',
                'grid_label_color': '#8f8f8f',
                'grid_label_size': 13,
                'line_width': 3,
                'threshold': true,
                'threshold_line_width': 2,
                'threshold_value': 90,
                'threshold_color': '#fd9600',
                'tension': 0.2,
                'segments': 32
            },
            'ring': {
                'background_color': '#c9d2d0',
                'line_width': 25,
                'line_width_secondary': 10,
                'labels': true,
                'label_percentage_size': 52,
                'label_percentage_color': '#343637',
                'label_title_size': 24,
                'label_title_color': '#343637',
                'label_title_spacing': 30,
                'line_cap': 'round'
            },
            'bar': {
                'threshold': true,
                'threshold_line_width': 2,
                'threshold_value': 90,
                'threshold_color': '#fd9600',
                'line_width': 20,
                'label_size': 20,
                'label_color': '#343637',
            },
            'bar_horizontal': {
                'height': 50,
                'gutter': 0,
                'budge': 20,
                'line_width': 20,
                'label_size': 20,
                'label_color': '#343637',
                'threshold': true,
                'threshold_line_width': 2,
                'threshold_value': 90,
                'threshold_color': '#fd9600'
            }
        };
    };
    /**
     * Check if there are any issues with graph options
     *
     * @param options
     * @private
     */
    AtajoXGraph.prototype._checkOptionErrors = function (options) {
        if (options.colors) {
            if (options.colors.length !== 2) {
                this.error('Only use two colours');
            }
        }
        if (options.type) {
            if (this.availableChartTypes.indexOf(options.type) === -1) {
                this.error('Only ' + this.availableChartTypes.length + ' types of charts are available: ' + this.availableChartTypes.join(', '));
            }
            if (options.type === 'line' || options.type === 'bar') {
                if (options.series) {
                    if (options.series.length === 0) {
                        this.error('You need to have series information');
                        return;
                    }
                }
                else {
                    this.error('You need to have a series array in your options');
                    return;
                }
                if (options.data) {
                    if (options.data.length === 0) {
                        this.error('You need to have data');
                        return;
                    }
                }
                else {
                    this.error('You need to have a data array in your options');
                    return;
                }
            }
            if (options.type === 'ring') {
                if (options.percentage) {
                    if (options.percentage !== parseInt(options.percentage)) {
                        this.error('Percentage should be an integer');
                        return;
                    }
                }
                else {
                    this.error('You need to have a percentage integer in your options');
                    return;
                }
            }
        }
    };
    /**
     * Set up axis
     *
     * @private
     */
    AtajoXGraph.prototype._axisSetup = function () {
        this._yAxisStartX = this._gutter + this._nudge;
        this._yAxisStartY = this._gutter + this._nudge;
        this._yAxisEndX = this._gutter + this._nudge;
        this._yAxisEndY = this._c.height - (this._gutter + this._nudge);
        this._xAxisStartX = this._gutter + this._nudge;
        this._xAxisStartY = this._c.height - (this._gutter + this._nudge);
        this._xAxisEndX = this._c.width - (this._gutter + this._nudge);
        this._xAxisEndY = this._c.height - (this._gutter + this._nudge);
        this._xAxisLength = this._xAxisEndX - this._xAxisStartX;
        this._yAxisLength = this._yAxisEndY - this._yAxisStartY;
        if (this._options.series.length === 1) {
            this._serieSpacing = this._xAxisLength / 2;
        }
        else {
            this._serieSpacing = this._xAxisLength / (this._options.series.length - 1);
        }
    };
    /**
     * Draw graph title
     *
     * @private
     */
    AtajoXGraph.prototype._drawTitle = function () {
        this._ctx.font = this._options.line.title_size + 'px Roboto';
        this._ctx.fillStyle = '#000000';
        this._ctx.textAlign = 'start';
        this._ctx.fillText(this._options.title, 10 * this._ratio, 15 * this._ratio);
    };
    /**
     * Draw graph grid
     *
     * @param options
     * @private
     */
    AtajoXGraph.prototype._drawGrid = function (options) {
        options = (typeof options === 'undefined') ? { xaxis: true, yaxis: true } : options;
        options.xaxis = (typeof options.xaxis === 'undefined') ? true : options.xaxis;
        options.yaxis = (typeof options.yaxis === 'undefined') ? true : options.yaxis;
        var minLineHeight = 90;
        // Y Axes
        if (options.yaxis) {
            this._ctx.beginPath();
            this._ctx.lineWidth = this._options.line.grid_line_width;
            this._ctx.strokeStyle = this._options.line.grid_color;
            this._ctx.moveTo(this._yAxisStartX, this._yAxisStartY);
            this._ctx.lineTo(this._yAxisEndX, this._yAxisEndY);
            this._ctx.stroke();
        }
        // X Axes
        if (options.xaxis) {
            this._ctx.beginPath();
            this._ctx.lineWidth = this._options.line.grid_line_width;
            this._ctx.strokeStyle = this._options.line.grid_color;
            this._ctx.moveTo(this._xAxisStartX, this._xAxisStartY);
            this._ctx.lineTo(this._xAxisEndX, this._xAxisEndY);
            this._ctx.stroke();
        }
        if (options.yaxis) {
            this._ctx.font = this._options.line.grid_label_size + 'px Roboto';
            this._ctx.fillStyle = this._options.line.grid_label_color;
            this._ctx.fillText('0%', this._gutter, this._c.height - (this._gutter + this._nudge));
            this._ctx.fillText('50%', this._gutter, this._gutter + this._nudge + (this._yAxisLength / 2));
            this._ctx.fillText('100%', this._gutter, this._gutter + this._nudge);
        }
        var thresholdXPositionAsY = 0;
        if (this._options.type === 'line' && this._options.line.threshold) {
            thresholdXPositionAsY = this._gutter + this._nudge + (this._yAxisLength - (this._options.line.threshold_value / 100) * this._yAxisLength);
            this._ctx.beginPath();
            this._ctx.lineWidth = this._options.line.threshold_line_width;
            this._ctx.strokeStyle = this._options.line.threshold_color;
            this._ctx.moveTo(this._gutter + this._nudge, thresholdXPositionAsY);
            this._ctx.lineTo(this._c.width - (this._gutter + this._nudge), thresholdXPositionAsY);
            this._ctx.stroke();
            if (this._yAxisLength > minLineHeight) {
                this._ctx.fillText(this._options.line.threshold_value + '%', this._gutter, thresholdXPositionAsY);
            }
        }
        if (this._options.type === 'bar' && this._options.bar.threshold) {
            thresholdXPositionAsY = this._gutter + this._nudge + (this._yAxisLength - (this._options.bar.threshold_value / 100) * this._yAxisLength);
            this._ctx.beginPath();
            this._ctx.lineWidth = this._options.bar.threshold_line_width;
            this._ctx.strokeStyle = this._options.bar.threshold_color;
            this._ctx.moveTo(this._gutter + this._nudge, thresholdXPositionAsY);
            this._ctx.lineTo(this._c.width - (this._gutter + this._nudge), thresholdXPositionAsY);
            this._ctx.stroke();
        }
    };
    /**
     * Draw X Series
     * @private
     */
    AtajoXGraph.prototype._drawXSeries = function () {
        if (this._options.series.length === 1) {
            this._drawXSerieMarker(this._yAxisStartX + this._serieSpacing, this._options.series[0]);
        }
        else {
            for (var i = 0; i < this._options.series.length; i++) {
                this._drawXSerieMarker(this._yAxisStartX + (this._serieSpacing * (i)), this._options.series[i]);
            }
        }
    };
    /**
     * Draw X Serie marker
     *
     * @param xPoint
     * @param label
     * @private
     */
    AtajoXGraph.prototype._drawXSerieMarker = function (xPoint, label) {
        var markerHeight = 3 * this._ratio;
        var labelMarkerSpacing = 8 * this._ratio;
        this._ctx.beginPath();
        this._ctx.strokeStyle = this._options.line.grid_color;
        this._ctx.moveTo(xPoint, this._yAxisEndY);
        this._ctx.lineTo(xPoint, this._yAxisEndY + markerHeight);
        this._ctx.stroke();
        this._ctx.font = this._options.line.grid_label_size + 'px Roboto';
        this._ctx.fillStyle = this._options.line.grid_label_color;
        this._ctx.textAlign = 'center';
        this._ctx.fillText(label, xPoint, this._yAxisEndY + markerHeight + labelMarkerSpacing);
    };
    /**
     * Get line curve points
     *
     * @param ptsa
     * @returns {Array<number>}
     * @private
     */
    AtajoXGraph.prototype._getLineCurvePoints = function (ptsa) {
        var tension = this._options.line.tension;
        var numOfSegments = this._options.line.segments;
        var res = [];
        var x;
        var y;
        var t1x;
        var t2x;
        var t1y;
        var t2y;
        var c1;
        var c2;
        var c3;
        var c4;
        var st;
        var t;
        var i;
        var _pts = ptsa.slice(0);
        _pts.unshift(ptsa[1]);
        _pts.unshift(ptsa[0]);
        _pts.push(ptsa[ptsa.length - 2]);
        _pts.push(ptsa[ptsa.length - 1]);
        for (i = 2; i < (_pts.length - 4); i += 2) {
            t1x = (_pts[i + 2] - _pts[i - 2]) * tension;
            t2x = (_pts[i + 4] - _pts[i]) * tension;
            t1y = (_pts[i + 3] - _pts[i - 1]) * tension;
            t2y = (_pts[i + 5] - _pts[i + 1]) * tension;
            for (t = 0; t <= numOfSegments; t++) {
                st = t / numOfSegments;
                c1 = 2 * Math.pow(st, 3) - 3 * Math.pow(st, 2) + 1;
                c2 = -(2 * Math.pow(st, 3)) + 3 * Math.pow(st, 2);
                c3 = Math.pow(st, 3) - 2 * Math.pow(st, 2) + st;
                c4 = Math.pow(st, 3) - Math.pow(st, 2);
                x = c1 * _pts[i] + c2 * _pts[i + 2] + c3 * t1x + c4 * t2x;
                y = c1 * _pts[i + 1] + c2 * _pts[i + 3] + c3 * t1y + c4 * t2y;
                res.push(x);
                res.push(y);
            }
        }
        if (this._options.animate) {
            return res.slice(0, this.steps);
        }
        else {
            return res;
        }
    };
    /**
     * Draw graph lines
     *
     * @param ctx
     * @param pts
     * @private
     */
    AtajoXGraph.prototype._drawLines = function (ctx, pts) {
        ctx.beginPath();
        ctx.moveTo(pts[0], pts[1]);
        for (var i = 2; i < pts.length - 1; i += 2) {
            ctx.lineTo(pts[i], pts[i + 1]);
        }
        ctx.lineWidth = this._options.line.line_width;
        ctx.strokeStyle = this._options.colors[0];
        ctx.lineCap = 'round';
        ctx.stroke();
    };
    /**
     * Draw area fill
     *
     * @param ctx
     * @param pts
     * @private
     */
    AtajoXGraph.prototype._drawAreaFill = function (ctx, pts) {
        if (this._options.line.grid_fill) {
            ctx.beginPath();
            pts.unshift(pts[0], this._xAxisStartY);
            pts.push(pts[pts.length - 2], this._xAxisStartY);
            ctx.moveTo(pts[0], pts[1]);
            for (var i = 2; i < pts.length - 1; i += 2) {
                ctx.lineTo(pts[i], pts[i + 1]);
            }
            ctx.closePath();
            ctx.fillStyle = 'rgba(240, 240, 240, 0.5)';
            ctx.fill();
        }
    };
    /**
     * Draw line chart
     *
     * @private
     */
    AtajoXGraph.prototype._drawLineChart = function () {
        if (this._options.title !== '') {
            this._drawTitle();
        }
        if (this._options.line.grid) {
            this._drawGrid();
        }
        if (this._options.line.grid_series) {
            this._drawXSeries();
        }
        if (this._options.data.length === 1) {
            this._ctx.beginPath();
            this._ctx.arc(this._yAxisStartX, this._yAxisStartY + this._yAxisLength - (this._options.data[0] / 100) * this._yAxisLength, 10, 0, 2 * Math.PI, false);
            this._ctx.fillStyle = this._options.colors[0];
            this._ctx.fill();
        }
        else {
            var points = [];
            for (var i = 0; i < this._options.data.length; i++) {
                points.push(this._yAxisStartX + (this._serieSpacing * i));
                points.push(this._yAxisStartY + this._yAxisLength - (this._options.data[i] / 100) * this._yAxisLength);
            }
            this._drawLines(this._ctx, this._getLineCurvePoints(points));
            this._drawAreaFill(this._ctx, this._getLineCurvePoints(points));
        }
    };
    /**
     * Draw ring chart
     *
     * @private
     */
    AtajoXGraph.prototype._drawRingChart = function () {
        var _nudge = 5 * this._ratio;
        this._ctx.beginPath();
        this._ctx.arc(this._c.width / 2, this._c.height / 2, (this._c.width > this._c.height) ? this._c.height / 2 - _nudge : this._c.width / 2 - _nudge, 0, 2 * Math.PI, true);
        this._ctx.lineWidth = this._options.ring.line_width;
        this._ctx.strokeStyle = this._options.ring.background_color;
        this._ctx.lineCap = this._options.ring.line_cap;
        this._ctx.stroke();
        var percentage = this._options.percentage;
        var ringAngle = this._percentageToRadians(percentage);
        this._ctx.beginPath();
        this._ctx.arc(this._c.width / 2, this._c.height / 2, (this._c.width > this._c.height) ? this._c.height / 2 - _nudge : this._c.width / 2 - _nudge, ringAngle.start, ringAngle.end);
        this._ctx.lineWidth = this._options.ring.line_width;
        this._ctx.strokeStyle = this._options.colors[0];
        this._ctx.lineCap = this._options.ring.line_cap;
        this._ctx.stroke();
        if (percentage > 100) {
            var overAngle = this._percentageToRadians(percentage - 100);
            this._ctx.beginPath();
            this._ctx.arc(this._c.width / 2, this._c.height / 2, (this._c.width > this._c.height) ? this._c.height / 2 - _nudge : this._c.width / 2 - _nudge, overAngle.start, overAngle.end);
            this._ctx.lineWidth = this._options.ring.line_width_secondary;
            this._ctx.strokeStyle = this._options.colors[1];
            this._ctx.lineCap = this._options.ring.line_cap;
            this._ctx.stroke();
        }
        if (this._options.ring.labels) {
            this._ctx.font = this._options.ring.label_title_size + 'px Roboto';
            this._ctx.fillStyle = '#343637';
            this._ctx.textAlign = 'center';
            this._ctx.fillText(this._options.title.toUpperCase(), this._c.width / 2, this._c.height / 2 + this._options.ring.label_title_spacing);
            this._ctx.font = this._options.ring.label_percentage_size + 'px Roboto';
            this._ctx.fillStyle = '#343637';
            this._ctx.textAlign = 'center';
            this._ctx.fillText(percentage + '%', this._c.width / 2, this._c.height / 2);
        }
    };
    /**
     * Convert percentage to radians
     *
     * @param percentage
     * @returns {iPercentageToRadians}
     * @private
     */
    AtajoXGraph.prototype._percentageToRadians = function (percentage) {
        var degrees = (percentage / 100) * 360;
        return {
            'start': Math.PI / 2 * -1,
            'end': (Math.PI / 2 * -1) - (degrees * (Math.PI / 180) * -1)
        };
    };
    /**
     * Draw bar chart
     *
     * @private
     */
    AtajoXGraph.prototype._drawBarChart = function () {
        if (this._options.title !== '') {
            this._drawTitle();
        }
        if (this._options.line.grid) {
            this._drawGrid({ yaxis: false });
        }
        if (this._options.line.grid_series) {
            this._drawXSeries();
        }
        this._drawBars();
    };
    /**
     * Draw bars
     *
     * @private
     */
    AtajoXGraph.prototype._drawBars = function () {
        for (var i = 0; i < this._options.data.length; i++) {
            this._ctx.beginPath();
            this._ctx.moveTo((this._xAxisStartX + (this._serieSpacing * i)), this._xAxisStartY);
            this._ctx.lineTo((this._xAxisStartX + (this._serieSpacing * i)), (this._xAxisStartY - (this._options.data[i] / 100) * this._yAxisLength));
            this._ctx.lineWidth = this._options.bar.line_width;
            this._ctx.strokeStyle = this._options.colors[0];
            this._ctx.lineCap = 'round';
            this._ctx.stroke();
            this._ctx.font = this._options.bar.label_size + 'px Roboto';
            this._ctx.fillStyle = this._options.bar.label_color;
            this._ctx.textAlign = 'center';
            this._ctx.fillText(this._options.data[i] + '%', (this._xAxisStartX + (this._serieSpacing * i)), (this._xAxisStartY - (this._options.data[i] / 100) * this._yAxisLength) - 15);
        }
    };
    /**
     * Draw horizontal bar chart
     *
     * @private
     */
    AtajoXGraph.prototype._drawBarHorizontalChart = function () {
        this._gutter = (this._options.bar_horizontal.gutter === null) ? this._gutter : this._options.bar_horizontal.gutter;
        // Backdrop
        this._ctx.fillStyle = 'rgba(240, 240, 240, 0.5)';
        this._ctx.fillRect(this._gutter, this._gutter, this._c.width - (this._gutter * 2), this._options.bar_horizontal.height);
        var budge = this._options.bar_horizontal.budge;
        var hundred = this._c.width - (budge * 3.5 + (this._options.bar_horizontal.label_size * 2.8));
        // Line
        this._ctx.beginPath();
        this._ctx.moveTo(budge + (this._options.bar_horizontal.line_width / 2), this._gutter + (this._options.bar_horizontal.height / 2));
        this._ctx.lineTo((budge + (this._options.bar_horizontal.line_width / 2) + ((this._options.percentage / 100)) * hundred), this._gutter + (this._options.bar_horizontal.height / 2));
        this._ctx.lineWidth = this._options.bar_horizontal.line_width;
        this._ctx.strokeStyle = this._options.colors[0];
        this._ctx.lineCap = 'round';
        this._ctx.stroke();
        // Threshold
        if (this._options.bar_horizontal.threshold) {
            this._ctx.beginPath();
            this._ctx.moveTo(budge + (this._options.bar_horizontal.line_width / 2) + (this._options.bar_horizontal.threshold_value / 100) * hundred, this._gutter);
            this._ctx.lineTo(budge + (this._options.bar_horizontal.line_width / 2) + (this._options.bar_horizontal.threshold_value / 100) * hundred, this._gutter + this._options.bar_horizontal.height);
            this._ctx.lineWidth = this._options.bar_horizontal.threshold_line_width;
            this._ctx.strokeStyle = this._options.bar_horizontal.threshold_color;
            this._ctx.lineCap = 'square';
            this._ctx.stroke();
        }
        // Percentage
        this._ctx.font = this._options.bar_horizontal.label_size + 'px Roboto';
        this._ctx.fillStyle = this._options.bar_horizontal.label_color;
        this._ctx.textAlign = 'right';
        this._ctx.fillText(this._options.percentage + '%', this._c.width - budge, (this._gutter + this._options.bar_horizontal.height / 2) + (this._options.bar_horizontal.label_size / 2.5));
    };
    /**
     * Animate graph
     *
     * @private
     */
    AtajoXGraph.prototype._animate = function () {
        if (this._options.animate) {
            var animationFrame = requestAnimationFrame(this._animate);
            if (this._options.type === 'line') {
                if (this.steps === ((this._options.data.length + 4) * this._options.line.segments)) {
                    cancelAnimationFrame(animationFrame);
                    return;
                }
                this._ctx.clearRect(0, 0, this._c.width, this._c.height);
                this._drawLineChart();
                this.steps += 2;
            }
            if (this._options.type === 'ring') {
                if (this.ringTruePercentage === null) {
                    this.ringTruePercentage = this._options.percentage;
                }
                if (this.steps <= this.ringTruePercentage) {
                    this._options.percentage = this.steps;
                }
                else {
                    cancelAnimationFrame(animationFrame);
                    return;
                }
                this._ctx.clearRect(0, 0, this._c.width, this._c.height);
                this._drawRingChart();
                this.steps++;
            }
            if (this._options.type === 'bar') {
                if (this.steps === ((this._options.data.length + 4) * this._options.line.segments)) {
                    cancelAnimationFrame(animationFrame);
                    return;
                }
                this._ctx.clearRect(0, 0, this._c.width, this._c.height);
                this._drawBarChart();
                this.steps += 2;
            }
        }
    };
    return AtajoXGraph;
}());
export { AtajoXGraph };
//# sourceMappingURL=graph.js.map